#!/bin/bash

declare QEMU_BIN="qemu-system-x86_64_patched"
declare NET='-device e1000e'
declare ACCEL='-enable-kvm'
declare MEMORY='8G'

declare -A DRIVES_FILES=(
["arch"]="-drive file=arch.img,format=raw" ["windows"]="-drive file=windows.img,format=raw" ["windows-cdrom"]="-cdrom Win10_22H2_EnglishInternational_x64v1.iso"
)

declare BIOS="OVMF.fd"

declare CPU_OPTIONS="host,hypervisor=off,kvm=off,kvm_pv_unhalt,rdtscp=off,rdpid=off,vmx=on,invtsc=on"
declare CPU_CORES_NUMBER=12

dmi_get ()
{
  echo $(sudo dmidecode -t $1 | grep -e ".*$2.*:" | cut -d ":" -f2 | xargs)
}

declare -a SMBIOS=(
-smbios type=0,vendor="$(dmi_get 0 Vendor)",date="$(dmi_get 0 Date)",version="$(dmi_get 0 Version)",release="$(dmi_get 0 BIOS)",uefi=on
-smbios type=1,manufacturer="$(dmi_get 1 Manufacturer)",product="$(dmi_get 1 Product)",version="$(dmi_get 1 Version)",serial="$(dmi_get 1 Serial)",uuid="$(dmi_get 1 UUID)",sku="$(dmi_get 1 SKU)",family="$(dmi_get 1 Family)"
-smbios type=2,manufacturer="$(dmi_get 2 Manufacturer)",product="$(dmi_get 2 Product)",version="$(dmi_get 2 Version)",serial="$(dmi_get 2 Serial)",asset="$(dmi_get 2 Asset)",location="$(dmi_get 2 Location)"
-smbios type=3,manufacturer="$(dmi_get 3 Manufacturer)",version="$(dmi_get 3 Version)",serial="$(dmi_get 3 Serial)",asset="$(dmi_get 3 Asset)",sku="$(dmi_get 3 SKU)"
-smbios type=4,sock_pfx="$(dmi_get 4 Sock)",manufacturer="$(dmi_get 4 Manufacturer)",version="$(dmi_get 4 Version)",serial="$(dmi_get 4 Serial)",asset="$(dmi_get 4 Asset)",part="$(dmi_get 4 Part)"
-smbios type=11,value="$(dmi_get 11 1)",value="$(dmi_get 11 2)",value="$(dmi_get 11 3)",value="$(dmi_get 11 4)",value="$(dmi_get 11 5)",value="$(dmi_get 11 6)"
-smbios type=17,loc_pfx="$(dmi_get 17 Locator | cut -d ' ' -f1-5)",bank="$(dmi_get 17 Locator | cut -d ' ' -f1-2)",manufacturer="$(dmi_get 17 Manufacturer | cut -d ' ' -f1)",serial="$(dmi_get 17 Serial | cut -d ' ' -f1)",asset="$(dmi_get 17 Asset | cut -d ' ' -f1)",part="$(dmi_get 17 Part | cut -d ' ' -f1)",speed=$(dmi_get 17 Speed | cut -d ' ' -f1)
)
echo "${SMBIOS[@]}"
$QEMU_BIN -M q35 -boot $BOOT_OPTION $ACCEL $NET $NETDEV -m $MEMORY ${DRIVES_FILES[windows]} -cpu $CPU_OPTIONS -smp $CPU_CORES_NUMBER,sockets=1,threads=2,cores=$(($CPU_CORES_NUMBER / 2)) -drive if=pflash,format=raw,file=$BIOS "${SMBIOS[@]}"

unset QEMU_BIN
unset BOOT_OPTION
unset ACCEL
unset MEMORY
unset DRIVES_FILES
unset BIOS
unset CPU_OPTIONS
unset CPU_CORES_NUMBER
unset SMBIOS
unset -f dmi_get
